"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var handlebars = require("handlebars");
var Type_1 = require("./Type");
var Naming_1 = require("./Naming");
var Renderer_1 = require("./Renderer");
var Support_1 = require("./Support");
var Source_1 = require("./Source");
var lodash_1 = require("lodash");
var DeclarationIR_1 = require("./DeclarationIR");
var TypeGraph_1 = require("./TypeGraph");
var ConvenienceRenderer = /** @class */ (function (_super) {
    __extends(ConvenienceRenderer, _super);
    function ConvenienceRenderer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._alphabetizeProperties = false;
        _this.addDependenciesForNamedType = function (type, named) {
            var dependencyNames = _this.namedTypeDependencyNames(type, named);
            for (var _i = 0, dependencyNames_1 = dependencyNames; _i < dependencyNames_1.length; _i++) {
                var dn = dependencyNames_1[_i];
                _this._globalNamespace.add(dn);
            }
        };
        _this.nameForTopLevel = function (type, name) {
            var maybeNamedType = _this.namedTypeToNameForTopLevel(type);
            var styledName;
            if (maybeNamedType !== undefined) {
                styledName = _this._namedTypeNamer.nameStyle(name);
            }
            else {
                styledName = _this.topLevelNameStyle(name);
            }
            var named = _this._globalNamespace.add(new Naming_1.FixedName(styledName));
            var dependencyNames = _this.topLevelDependencyNames(type, named);
            for (var _i = 0, dependencyNames_2 = dependencyNames; _i < dependencyNames_2.length; _i++) {
                var dn = dependencyNames_2[_i];
                _this._globalNamespace.add(dn);
            }
            if (maybeNamedType !== undefined) {
                _this.addDependenciesForNamedType(maybeNamedType, named);
                _this._nameStoreView.set(maybeNamedType, named);
            }
            return named;
        };
        _this.addNamedForNamedType = function (type) {
            var existing = _this._nameStoreView.tryGet(type);
            if (existing !== undefined)
                return existing;
            var named = _this._globalNamespace.add(new Naming_1.SimpleName(type.getProposedNames(), _this._namedTypeNamer));
            _this.addDependenciesForNamedType(type, named);
            _this._nameStoreView.set(type, named);
            return named;
        };
        _this.addPropertyNames = function (c, className) {
            var propertyNamer = _this._classPropertyNamer;
            if (propertyNamer === null)
                return;
            var _a = _this.processForbiddenWordsInfo(_this.forbiddenForClassProperties(c, className), "forbidden-for-properties"), forbiddenNames = _a.forbiddenNames, forbiddenNamespaces = _a.forbiddenNamespaces;
            var ns = new Naming_1.Namespace(c.getCombinedName(), _this._globalNamespace, forbiddenNamespaces, forbiddenNames);
            var names = c.sortedProperties
                .map(function (_, name) {
                // FIXME: This alternative should really depend on what the
                // actual name of the class ends up being.  We can do this
                // with a DependencyName.
                // Also, we currently don't have any languages where properties
                // are global, so collisions here could only occur where two
                // properties of the same class have the same name, in which case
                // the alternative would also be the same, i.e. useless.  But
                // maybe we'll need global properties for some weird language at
                // some point.
                var alternative = c.getCombinedName() + "_" + name;
                return ns.add(new Naming_1.SimpleName(immutable_1.OrderedSet([name, alternative]), propertyNamer));
            })
                .toMap();
            _this._propertyNamesStoreView.set(c, names);
        };
        _this.addUnionMemberNames = function (u, unionName) {
            var memberNamer = _this._unionMemberNamer;
            if (memberNamer === null)
                return;
            var _a = _this.processForbiddenWordsInfo(_this.forbiddenForUnionMembers(u, unionName), "forbidden-for-union-members"), forbiddenNames = _a.forbiddenNames, forbiddenNamespaces = _a.forbiddenNamespaces;
            var ns;
            if (_this.unionMembersInGlobalNamespace) {
                ns = _this._globalNamespace;
            }
            else {
                ns = new Naming_1.Namespace(u.getCombinedName(), _this._globalNamespace, forbiddenNamespaces, forbiddenNames);
            }
            var names = immutable_1.Map();
            u.members.forEach(function (t) {
                var name = _this.makeUnionMemberName(u, unionName, t);
                names = names.set(t, ns.add(name));
            });
            _this._memberNamesStoreView.set(u, names);
        };
        // FIXME: this is very similar to addPropertyNameds and addUnionMemberNames
        _this.addEnumCaseNames = function (e, enumName) {
            var caseNamer = _this._enumCaseNamer;
            if (caseNamer === null)
                return;
            var _a = _this.processForbiddenWordsInfo(_this.forbiddenForEnumCases(e, enumName), "forbidden-for-enum-cases"), forbiddenNames = _a.forbiddenNames, forbiddenNamespaces = _a.forbiddenNamespaces;
            var ns;
            if (_this.enumCasesInGlobalNamespace) {
                ns = _this._globalNamespace;
            }
            else {
                ns = new Naming_1.Namespace(e.getCombinedName(), _this._globalNamespace, forbiddenNamespaces, forbiddenNames);
            }
            var names = immutable_1.Map();
            e.cases.forEach(function (name) {
                // FIXME: See the FIXME in `addPropertyNameds`.  We do have global
                // enum cases, though (in Go), so this is actually useful already.
                var alternative = e.getCombinedName() + "_" + name;
                names = names.set(name, ns.add(new Naming_1.SimpleName(immutable_1.OrderedSet([name, alternative]), caseNamer)));
            });
            _this._caseNamesStoreView.set(e, names);
        };
        _this.childrenOfType = function (t) {
            var names = _this.names;
            if (t instanceof Type_1.ClassType && _this._classPropertyNamer !== null) {
                var propertyNameds_1 = _this._propertyNamesStoreView.get(t);
                var sortedMap = t.properties
                    .map(function (p) { return p.type; })
                    .sortBy(function (_, n) { return Support_1.defined(names.get(Support_1.defined(propertyNameds_1.get(n)))); });
                return sortedMap.toOrderedSet();
            }
            return t.children.toOrderedSet();
        };
        _this.proposedUnionMemberNameForTypeKind = function (_kind) {
            return null;
        };
        _this.nameForNamedType = function (t) {
            return _this._nameStoreView.get(t);
        };
        _this.forEachTopLevel = function (blankLocations, f, predicate) {
            var topLevels;
            if (predicate !== undefined) {
                topLevels = _this.topLevels.filter(predicate);
            }
            else {
                topLevels = _this.topLevels;
            }
            _this.forEachWithBlankLines(topLevels, blankLocations, function (t, name) {
                return f(t, _this._nameStoreView.getForTopLevel(name));
            });
        };
        _this.setAlphabetizeProperties = function (value) {
            _this._alphabetizeProperties = value;
        };
        _this.forEachClassProperty = function (c, blankLocations, f) {
            var propertyNames = _this._propertyNamesStoreView.get(c);
            if (_this._alphabetizeProperties) {
                var alphabetizedPropertyNames = propertyNames.sortBy(function (n) { return _this.names.get(n); }).toOrderedMap();
                _this.forEachWithBlankLines(alphabetizedPropertyNames, blankLocations, function (name, jsonName) {
                    var p = Support_1.defined(c.properties.get(jsonName));
                    f(name, jsonName, p);
                });
            }
            else {
                _this.forEachWithBlankLines(c.properties, blankLocations, function (p, jsonName) {
                    var name = Support_1.defined(propertyNames.get(jsonName));
                    f(name, jsonName, p);
                });
            }
        };
        _this.nameForUnionMember = function (u, t) {
            return Support_1.defined(_this._memberNamesStoreView.get(u).get(t));
        };
        _this.forEachUnionMember = function (u, members, blankLocations, sortOrder, f) {
            var iterateMembers = members === null ? u.members : members;
            if (sortOrder === null) {
                sortOrder = function (n) { return Support_1.defined(_this.names.get(n)); };
            }
            var memberNames = _this._memberNamesStoreView.get(u).filter(function (_, t) { return iterateMembers.has(t); });
            var sortedMemberNames = memberNames.sortBy(sortOrder).toOrderedMap();
            _this.forEachWithBlankLines(sortedMemberNames, blankLocations, f);
        };
        _this.forEachEnumCase = function (e, blankLocations, f) {
            var caseNames = _this._caseNamesStoreView.get(e);
            var sortedCaseNames = caseNames.sortBy(function (n) { return _this.names.get(n); }).toOrderedMap();
            _this.forEachWithBlankLines(sortedCaseNames, blankLocations, f);
        };
        _this.forEachClass = function (blankLocations, f) {
            _this.forEachSpecificNamedType(blankLocations, _this._namedClasses, f);
        };
        _this.forEachEnum = function (blankLocations, f) {
            _this.forEachSpecificNamedType(blankLocations, _this._namedEnums, f);
        };
        _this.forEachUnion = function (blankLocations, f) {
            _this.forEachSpecificNamedType(blankLocations, _this._namedUnions, f);
        };
        _this.forEachNamedType = function (blankLocations, classFunc, enumFunc, unionFunc) {
            _this.forEachWithBlankLines(_this._namedTypes, blankLocations, function (t) {
                if (t instanceof Type_1.ClassType) {
                    _this.callForNamedType(t, classFunc);
                }
                else if (t instanceof Type_1.EnumType) {
                    _this.callForNamedType(t, enumFunc);
                }
                else if (t instanceof Type_1.UnionType) {
                    _this.callForNamedType(t, unionFunc);
                }
                else {
                    return Support_1.panic("Named type that's neither a class nor union");
                }
            });
        };
        // You should never have to use this to produce parts of your generated
        // code.  If you need to modify a Name, for example to change its casing,
        // use `modifySource`.
        _this.sourcelikeToString = function (src) {
            return Source_1.serializeRenderResult(Source_1.sourcelikeToSource(src), _this.names, "").lines.join("\n");
        };
        _this.emitCommentLines = function (commentStart, lines) {
            for (var _i = 0, lines_1 = lines; _i < lines_1.length; _i++) {
                var line = lines_1[_i];
                _this.emitLine(lodash_1.trimEnd(commentStart + line));
            }
        };
        return _this;
    }
    Object.defineProperty(ConvenienceRenderer.prototype, "topLevels", {
        get: function () {
            return this.typeGraph.topLevels;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "forbiddenNamesForGlobalNamespace", {
        get: function () {
            return [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "forbiddenNamesForClassProperties", {
        get: function () {
            return undefined;
        },
        enumerable: true,
        configurable: true
    });
    ConvenienceRenderer.prototype.forbiddenForClassProperties = function (_c, _className) {
        return { names: [], includeGlobalForbidden: false };
    };
    ConvenienceRenderer.prototype.forbiddenForUnionMembers = function (_u, _unionName) {
        return { names: [], includeGlobalForbidden: false };
    };
    ConvenienceRenderer.prototype.forbiddenForEnumCases = function (_e, _enumName) {
        return { names: [], includeGlobalForbidden: false };
    };
    ConvenienceRenderer.prototype.topLevelDependencyNames = function (_t, _topLevelName) {
        return [];
    };
    ConvenienceRenderer.prototype.namedTypeDependencyNames = function (_t, _name) {
        return [];
    };
    ConvenienceRenderer.prototype.namedTypeToNameForTopLevel = function (type) {
        if (Type_1.isNamedType(type)) {
            return type;
        }
        return undefined;
    };
    Object.defineProperty(ConvenienceRenderer.prototype, "unionMembersInGlobalNamespace", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "enumCasesInGlobalNamespace", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "needsTypeDeclarationBeforeUse", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    ConvenienceRenderer.prototype.canBeForwardDeclared = function (_t) {
        return Support_1.panic("If needsTypeDeclarationBeforeUse returns true, canBeForwardDeclared must be implemented");
    };
    ConvenienceRenderer.prototype.unionNeedsName = function (u) {
        return Type_1.nullableFromUnion(u) === null;
    };
    ConvenienceRenderer.prototype.setUpNaming = function () {
        var _this = this;
        this._nameStoreView = new TypeGraph_1.TypeAttributeStoreView(this.typeGraph.attributeStore, "assignedName");
        this._propertyNamesStoreView = new TypeGraph_1.TypeAttributeStoreView(this.typeGraph.attributeStore, "assignedPropertyNames");
        this._memberNamesStoreView = new TypeGraph_1.TypeAttributeStoreView(this.typeGraph.attributeStore, "assignedMemberNames");
        this._caseNamesStoreView = new TypeGraph_1.TypeAttributeStoreView(this.typeGraph.attributeStore, "assignedCaseNames");
        this._namedTypeNamer = this.makeNamedTypeNamer();
        this._classPropertyNamer = this.makeClassPropertyNamer();
        this._unionMemberNamer = this.makeUnionMemberNamer();
        this._enumCaseNamer = this.makeEnumCaseNamer();
        this._globalForbiddenNamespace = Naming_1.keywordNamespace("forbidden", this.forbiddenNamesForGlobalNamespace);
        this._otherForbiddenNamespaces = immutable_1.Map();
        this._globalNamespace = new Naming_1.Namespace("global", undefined, immutable_1.Set([this._globalForbiddenNamespace]), immutable_1.Set());
        var _a = this.typeGraph.allNamedTypesSeparated(), classes = _a.classes, enums = _a.enums, unions = _a.unions;
        var namedUnions = unions.filter(function (u) { return _this.unionNeedsName(u); }).toOrderedSet();
        this.topLevels.forEach(function (t, name) {
            _this._nameStoreView.setForTopLevel(name, _this.nameForTopLevel(t, name));
        });
        classes.forEach(function (c) {
            var name = _this.addNamedForNamedType(c);
            _this.addPropertyNames(c, name);
        });
        enums.forEach(function (e) {
            var name = _this.addNamedForNamedType(e);
            _this.addEnumCaseNames(e, name);
        });
        namedUnions.forEach(function (u) {
            var name = _this.addNamedForNamedType(u);
            _this.addUnionMemberNames(u, name);
        });
        return immutable_1.OrderedSet([this._globalForbiddenNamespace, this._globalNamespace]).union(this._otherForbiddenNamespaces.valueSeq());
    };
    ConvenienceRenderer.prototype.processForbiddenWordsInfo = function (info, namespaceName) {
        var forbiddenNames = [];
        var forbiddenStrings = [];
        for (var _i = 0, _a = info.names; _i < _a.length; _i++) {
            var nameOrString = _a[_i];
            if (typeof nameOrString === "string") {
                forbiddenStrings.push(nameOrString);
            }
            else {
                forbiddenNames.push(nameOrString);
            }
        }
        var namespace = this._otherForbiddenNamespaces.get(namespaceName);
        if (forbiddenStrings.length > 0 && namespace === undefined) {
            namespace = Naming_1.keywordNamespace(namespaceName, forbiddenStrings);
            this._otherForbiddenNamespaces = this._otherForbiddenNamespaces.set(namespaceName, namespace);
        }
        var forbiddenNamespaces = immutable_1.Set();
        if (info.includeGlobalForbidden) {
            forbiddenNamespaces = forbiddenNamespaces.add(this._globalForbiddenNamespace);
        }
        if (namespace !== undefined) {
            forbiddenNamespaces = forbiddenNamespaces.add(namespace);
        }
        return { forbiddenNames: immutable_1.Set(forbiddenNames), forbiddenNamespaces: forbiddenNamespaces };
    };
    ConvenienceRenderer.prototype.makeUnionMemberName = function (u, unionName, t) {
        var _this = this;
        return new Naming_1.DependencyName(Support_1.nonNull(this._unionMemberNamer), function (lookup) {
            return _this.proposeUnionMemberName(u, unionName, t, lookup);
        });
    };
    Object.defineProperty(ConvenienceRenderer.prototype, "namedUnions", {
        get: function () {
            return this._namedUnions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "haveNamedUnions", {
        get: function () {
            return !this._namedUnions.isEmpty();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "haveUnions", {
        get: function () {
            return this._haveUnions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "haveOptionalProperties", {
        get: function () {
            return this._haveOptionalProperties;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "enums", {
        get: function () {
            return this._namedEnums;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConvenienceRenderer.prototype, "haveEnums", {
        get: function () {
            return !this._namedEnums.isEmpty();
        },
        enumerable: true,
        configurable: true
    });
    ConvenienceRenderer.prototype.proposeUnionMemberName = function (_u, _unionName, fieldType, lookup) {
        var _this = this;
        var simpleName = this.proposedUnionMemberNameForTypeKind(fieldType.kind);
        if (simpleName !== null) {
            return simpleName;
        }
        var typeNameForUnionMember = function (t) {
            return Type_1.matchTypeExhaustive(t, function (_noneType) {
                return Support_1.panic("None type should have been replaced");
            }, function (_anyType) { return "anything"; }, function (_nullType) { return "null"; }, function (_boolType) { return "bool"; }, function (_integerType) { return "integer"; }, function (_doubleType) { return "double"; }, function (_stringType) { return "string"; }, function (arrayType) { return typeNameForUnionMember(arrayType.items) + "_array"; }, function (classType) { return lookup(_this.nameForNamedType(classType)); }, function (mapType) { return typeNameForUnionMember(mapType.values) + "_map"; }, function (_enumType) { return "enum"; }, function (_unionType) { return "union"; }, function (_dateType) { return "date"; }, function (_timeType) { return "time"; }, function (_dateTimeType) { return "date_time"; });
        };
        return typeNameForUnionMember(fieldType);
    };
    ConvenienceRenderer.prototype.isForwardDeclaredType = function (t) {
        return this._declarationIR.forwardedTypes.has(t);
    };
    ConvenienceRenderer.prototype.isImplicitCycleBreaker = function (_t) {
        return Support_1.panic("A renderer that invokes isCycleBreakerType must implement canBeCycleBreakerType");
    };
    ConvenienceRenderer.prototype.canBreakCycles = function (_t) {
        return true;
    };
    ConvenienceRenderer.prototype.isCycleBreakerType = function (t) {
        var _this = this;
        if (this._cycleBreakerTypes === undefined) {
            this._cycleBreakerTypes = DeclarationIR_1.cycleBreakerTypesForGraph(this.typeGraph, function (s) { return _this.isImplicitCycleBreaker(s); }, function (s) { return _this.canBreakCycles(s); });
        }
        return this._cycleBreakerTypes.has(t);
    };
    ConvenienceRenderer.prototype.forEachDeclaration = function (blankLocations, f) {
        this.forEachWithBlankLines(this._declarationIR.declarations, blankLocations, f);
    };
    ConvenienceRenderer.prototype.callForNamedType = function (t, f) {
        f(t, this.nameForNamedType(t));
    };
    ConvenienceRenderer.prototype.forEachSpecificNamedType = function (blankLocations, types, f) {
        var _this = this;
        this.forEachWithBlankLines(types, blankLocations, function (t) {
            _this.callForNamedType(t, f);
        });
    };
    ConvenienceRenderer.prototype.forEachUniqueUnion = function (blankLocations, uniqueValue, f) {
        var firstUnionByValue = immutable_1.OrderedMap();
        this._namedUnions.forEach(function (u) {
            var v = uniqueValue(u);
            if (!firstUnionByValue.has(v)) {
                firstUnionByValue = firstUnionByValue.set(v, u);
            }
        });
        this.forEachWithBlankLines(firstUnionByValue, blankLocations, f);
    };
    ConvenienceRenderer.prototype.processGraph = function () {
        var _this = this;
        this._declarationIR = DeclarationIR_1.declarationsForGraph(this.typeGraph, this.needsTypeDeclarationBeforeUse ? function (t) { return _this.canBeForwardDeclared(t); } : undefined, this.childrenOfType, function (t) {
            if (t instanceof Type_1.UnionType) {
                return _this.unionNeedsName(t);
            }
            return Type_1.isNamedType(t);
        });
        var types = this.typeGraph.allTypesUnordered();
        this._haveUnions = types.some(function (t) { return t instanceof Type_1.UnionType; });
        this._haveOptionalProperties = types
            .filter(function (t) { return t instanceof Type_1.ClassType; })
            .some(function (c) { return c.properties.some(function (p) { return p.isOptional; }); });
        this._namedTypes = this._declarationIR.declarations.filter(function (d) { return d.kind === "define"; }).map(function (d) { return d.type; });
        var _a = Type_1.separateNamedTypes(this._namedTypes), classes = _a.classes, enums = _a.enums, unions = _a.unions;
        this._namedClasses = classes;
        this._namedEnums = enums;
        this._namedUnions = unions;
    };
    ConvenienceRenderer.prototype.emitSource = function (givenOutputFilename) {
        this.processGraph();
        this.emitSourceStructure(givenOutputFilename);
    };
    ConvenienceRenderer.prototype.makeHandlebarsContextForUnionMember = function (t, name) {
        var value = this.makeHandlebarsContextForType(t);
        value.assignedName = Support_1.defined(this.names.get(name));
        return value;
    };
    ConvenienceRenderer.prototype.makeHandlebarsContextForType = function (t) {
        var value = { type: { kind: t.kind, index: t.typeRef.getIndex() } };
        var maybeName = this._nameStoreView.tryGet(t);
        if (maybeName !== undefined) {
            value.assignedName = this.names.get(maybeName);
        }
        return value;
    };
    ConvenienceRenderer.prototype.makeHandlebarsContext = function () {
        var _this = this;
        this.processGraph();
        var allTypes = [];
        this.typeGraph.allTypesUnordered().forEach(function (t) {
            var value = _this.makeHandlebarsContextForType(t);
            if (t instanceof Type_1.ClassType) {
                var properties_1 = {};
                _this.forEachClassProperty(t, "none", function (name, jsonName, p) {
                    var propertyValue = _this.makeHandlebarsContextForType(p.type);
                    propertyValue.isOptional = p.isOptional;
                    propertyValue.assignedName = Support_1.defined(_this.names.get(name));
                    properties_1[jsonName] = propertyValue;
                });
                value.properties = properties_1;
            }
            else if (t instanceof Type_1.EnumType) {
                var cases_1 = {};
                _this.forEachEnumCase(t, "none", function (name, jsonName) {
                    cases_1[jsonName] = { assignedName: Support_1.defined(_this.names.get(name)) };
                });
                value.cases = cases_1;
            }
            else if (t instanceof Type_1.UnionType) {
                var members_1 = [];
                // FIXME: It's a bit ugly to have these two cases.
                if (_this._memberNamesStoreView.tryGet(t) === undefined) {
                    t.members.forEach(function (m) {
                        members_1.push(_this.makeHandlebarsContextForType(m));
                    });
                }
                else {
                    _this.forEachUnionMember(t, null, "none", null, function (name, m) {
                        members_1.push(_this.makeHandlebarsContextForUnionMember(m, name));
                    });
                }
                value.members = members_1;
            }
            var index = t.typeRef.getIndex();
            while (allTypes.length <= index) {
                allTypes.push(undefined);
            }
            allTypes[index] = value;
        });
        var namedTypes = [];
        var addNamedType = function (t) {
            namedTypes.push(allTypes[t.typeRef.getIndex()]);
        };
        this.forEachNamedType("none", addNamedType, addNamedType, addNamedType);
        var topLevels = {};
        this.topLevels.forEach(function (t, name) {
            var value = allTypes[t.typeRef.getIndex()];
            value.assignedTopLevelName = _this.names.get(_this._nameStoreView.getForTopLevel(name));
            topLevels[name] = value;
        });
        return { allTypes: allTypes, topLevels: topLevels, namedTypes: namedTypes };
    };
    ConvenienceRenderer.prototype.registerHandlebarsHelpers = function (context) {
        _super.prototype.registerHandlebarsHelpers.call(this, context);
        handlebars.registerHelper("with_type", function (t, options) {
            return options.fn(context.allTypes[t.index]);
        });
    };
    return ConvenienceRenderer;
}(Renderer_1.Renderer));
exports.ConvenienceRenderer = ConvenienceRenderer;
